%%%%
%Fitting code for sub-epidemic model associated with paper "A novel sub-epidemic modeling framework for short-term forecasting epidemic waves"
%Chowell et al. BMC Medicine 2019
%%%%

clear
close all

global invasions

for outbreak1=3
    
    close all
    
    switch outbreak1
        
        case 1
            
            datafilename1='curve-plague-Madagascar-wave2.txt';
            DT=7;
            
        case 2
            
            datafilename1='Ebola-DRC-April9.txt';
            DT=7;
 
            
        case 3
            datafilename1='SARS-Singapore.txt';
            DT=1;
    end
    
    
    M=300; % number of epidemic simulations
    
    SSstats0=[];
    SSstats1=[];
    SSstats2=[];
    
    rmses0=[];
    rmses1=[];
    rmses2=[];
    
    for flag1=[1] %1=GLM
        
        
        close all
        
        data=load(datafilename1);
        
        plot(data(:,1),data(:,2),'ko')
        
        %pause
        
        [max1,index1]=max(data(:,2));
        
        step1=1;
        
        t_start1=6;
        
        t_end1=index1;
        
        scalar1=1;
        
        param_r_time=[];
        param_p_time=[];
        param_a_time=[];
        param_K_time=[];
        
        param_q_time=[];
        
        %for i=[tfss(1:end) length(data(:,1))]
        
        for i=[length(data(:,1))]
            
            t_window=1:i;
            
            t_window
            
            data1=data(t_window',:)
            
            plot(data1(:,1),data1(:,2),'ko')
            
            epidemic_period=t_window;
            
            [param_r,param_p,param_a,param_K,param_q,Phatss,npatches,onset_thr,curves,fittedcurves,bestfit,data1]=fittingModifiedLogisticFunctionPatch(datafilename1,DT,t_window,M,scalar1,flag1)
            
            param_r_time=[param_r_time;[i param_r]];
            param_p_time=[param_p_time;[i param_p]];
            param_a_time=[param_a_time;[i param_a]];
            param_K_time=[param_K_time;[i param_K]];
            param_q_time=[param_q_time;[i param_q]];
            
            
            save(strcat('modifiedLogisticPatch-trained-',num2str(i),'-',datafilename1(1:end-4),'-flag-',num2str(flag1),'.mat'),'-mat')
            
        end
        
        
        
        %% plot best fit and parameter uncertainties
        
        param_r=[mean(Phatss(:,1)) plims(Phatss(:,1),0.025) plims(Phatss(:,1),0.975)];
        
        param_p=[mean(Phatss(:,2)) plims(Phatss(:,2),0.025) plims(Phatss(:,2),0.975)];
        
        param_a=[mean(Phatss(:,3)) plims(Phatss(:,3),0.025) plims(Phatss(:,3),0.975)];
        
        param_K=[mean(Phatss(:,4)) plims(Phatss(:,4),0.025) plims(Phatss(:,4),0.975)];
        
        param_q=[mean(Phatss(:,5)) plims(Phatss(:,5),0.025) plims(Phatss(:,5),0.975)];
        
        
        cad1=strcat('r=',num2str(param_r(end,1),2),'(95%CI:',num2str(param_r(end,2),2),',',num2str(param_r(end,3),2),')')
        cad2=strcat('p=',num2str(param_p(end,1),2),'(95%CI:',num2str(param_p(end,2),2),',',num2str(param_p(end,3),2),')')
        cad3=strcat('a=',num2str(param_a(end,1),2),'(95%CI:',num2str(param_a(end,2),2),',',num2str(param_a(end,3),2),')')
        cad4=strcat('K=',num2str(param_K(end,1),3),'(95%CI:',num2str(param_K(end,2),3),',',num2str(param_K(end,3),3),')')
        cad5=strcat('q=',num2str(param_q(end,1),3),'(95%CI:',num2str(param_q(end,2),3),',',num2str(param_q(end,3),3),')')
        
        
        close all
        figure(100)
        timevect=(data1(:,1))*DT;
        
        subplot(2,4,1)
        hist(Phatss(:,1))
        xlabel('r')
        ylabel('Frequency')
        title(cad1)
        
        hold on
        
        line2=[param_r(1,2) 10;param_r(1,3) 10];
        
        line1=plot(line2(:,1),line2(:,2),'r--')
        set(line1,'LineWidth',2)
        
        axis([param_r(1,2)-0.05 param_r(1,3)+0.05 0 80])
        
        set(gca,'FontSize', 16);
        set(gcf,'color','white')
        
        
        subplot(2,4,2)
        hist(Phatss(:,2))
        xlabel('p')
        hold on
        title(cad2)
        
        line2=[param_p(1,2) 10;param_p(1,3) 10];
        
        line1=plot(line2(:,1),line2(:,2),'r--')
        set(line1,'LineWidth',2)
        
        axis([param_p(1,2)-0.025 param_p(1,3)+0.025 0 80])
        
        set(gca,'FontSize', 16);
        set(gcf,'color','white')
        
        
        subplot(2,4,3)
        hist(Phatss(:,4))
        xlabel('K')
        hold on
        title(cad4)
        
        line2=[param_K(1,2) 10;param_K(1,3) 10];
        
        line1=plot(line2(:,1),line2(:,2),'r--')
        set(line1,'LineWidth',2)
        
        axis([param_K(1,2)-50 param_K(1,3)+50 0 80])
        
        set(gca,'FontSize', 16);
        set(gcf,'color','white')
        
        subplot(2,4,4)
        hist(Phatss(:,5))
        xlabel('q')
        hold on
        title(cad5)
        
        line2=[param_q(1,2) 10;param_q(1,3) 10];
        
        line1=plot(line2(:,1),line2(:,2),'r--')
        set(line1,'LineWidth',2)
        
        axis([param_q(1,2)-0.05 param_q(1,3)+0.05 0 80])
        
        set(gca,'FontSize', 16);
        set(gcf,'color','white')
        
        
        % plot model fit and uncertainty
        
        
        subplot(2,4,[5 6 7 8])
        plot(timevect,curves,'c-')
        hold on
        
        incidence1=bestfit;
        
        line1=plot(timevect,mean(curves,2),'r-')
        set(line1,'LineWidth',2)
        
        line1=plot(timevect,plims(curves',0.025),'r--')
        set(line1,'LineWidth',2)
        
        line1=plot(timevect,plims(curves',0.975),'r--')
        set(line1,'LineWidth',2)
        
        line1=plot(timevect,data1(:,2),'bo')
        set(line1,'LineWidth',2)
        
        hold on
        
        set(gca,'FontSize', 16);
        set(gcf,'color','white')
        
        %goodness of fit
        
        SSE=sum((mean(curves,2)-data1(:,2)).^2);
        
    end % flag1
    
end % outbreak1

