%%%%
%Fitting code for sub-epidemic model associated with paper "A novel sub-epidemic modeling framework for short-term forecasting epidemic waves"
%Chowell et al. BMC Medicine 2019
%%%%

function [param_r,param_p,param_a,param_K,param_q,Phatss,npatches,onset_thr,curves,fittedcurves,bestfit,data1]=fittingModifiedLogisticFunction(datafilename1,DT,epidemic_period,M,scalar1,flag1)

global invasions

close all

%%

% fitting

data1=load(datafilename1);

data1=data1(epidemic_period,:);

data1(:,2)=data1(:,2)/scalar1;

I0=data1(1,2); % initial condition

if I0==0
    data1=data1(2:end,:);
end

data=data1(:,2);

[max1,index1]=max(data);

timevect=(data1(:,1))*DT;

% parameter guess
r=0.2;

if flag1==3 %logistic model (p=1)
    p=1;
else
    p=0.9;
end

a=1;
K=sum(data1(:,2));

q=0.3;

P0=[r p a K q];


I0=data(1); % initial condition


[max1,index1]=max(data1(:,2));

cum1=sum(data1(1:index1,2));

cum1=round(cum1);

onset_thrs=unique(cumsum(data1(1:index1,2)));

index2=find(onset_thrs<=0.8*cum1);

onset_thrs=onset_thrs(index2)';


RMSES=[];
PS=[];

options = optimoptions('lsqcurvefit','UseParallel',true,...
    'TolX',10^(-5),'TolFun',10^(-5));

npatchess=(1:1:min(18,ceil((timevect(end)/14)+1)));


for npatches=npatchess
    for onset_thr=onset_thrs
        
        r=P0(1);
        p=P0(2);
        a=P0(3);
        K=P0(4);
        q=P0(5);
        
        z(1)=r;
        z(2)=p;
        z(3)=a;
        z(4)=K;
        z(5)=q;
        
        
        switch flag1
            
            case 1 %GLM
                LB=[0  0 1 20 0];
                UB=[10  1 1 100000000 5];
            case 2 %GRM
                LB=[0  0 0 20 0];
                UB=[10  1 10 100000000  5];
            case 3 %Logistic
                LB=[0  1 1 20 0];
                UB=[10  1 1 100000000 5];
            case 4 %Richards
                LB=[0  1 0 20 0];
                UB=[10  1 10 100000000 5];
                
        end
        
        [P,resnorm,residual,exitflag,output,lambda,J]=lsqcurvefit(@plotModifiedLogisticGrowthPatch1,z,timevect,data,LB,UB,options,I0,npatches,onset_thr,flag1);
        
        q=P(5);
        
        RMSES=[RMSES;[npatches onset_thr q sqrt(sum(residual.^2))]];
        
        PS=[PS;P];
        
        
    end
end

reals=length(npatchess)*length(onset_thrs);


[RMSES,index1]=sortrows(RMSES,4);

PS=PS(index1,:);


% plot best fit

[RMSE1, index1]=min(RMSES(:,4));

npatches=RMSES(index1,1);

onset_thr=RMSES(index1,2);

q=RMSES(index1,3);


r_hat=PS(index1,1)
p_hat=PS(index1,2)
a_hat=PS(index1,3)
K_hat=PS(index1,4)
q_hat=PS(index1,5)

P=PS(index1,:);

IC=zeros(npatches,1);

IC(1,1)=I0;
IC(2:end,1)=1;


invasions=zeros(npatches,1);

invasions(1)=1;



[~,x]=ode23s(@modifiedLogisticGrowthPatch,timevect,IC,[],r_hat,p_hat,a_hat,K_hat,npatches,onset_thr,q_hat,flag1);

figure(100)

for j=1:npatches
    
    incidence1=[x(1,j);diff(x(:,j))];
    
    plot(timevect,incidence1)
    hold on
    
end

y=sum(x,2);

totinc=[y(1,1);diff(y(:,1))];

totinc(1)=totinc(1)-(npatches-1);

bestfit=totinc;

plot(timevect,totinc,'r')

hold on
plot(timevect,data,'bo')
xlabel('Time (days)');
ylabel('Data and best SIR fit')

title('best fit')
[npatches onset_thr]


[npatches onset_thr q_hat]

PS(index1,:)

pause(1)

%

figure(101)


subplot(4,2,1)

semilogy(1:reals,RMSES(:,4),'x')
xlabel('Realization')
ylabel('RMSE')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
semilogy(index1,RMSE1,'ro')

title(strcat('RMSE=',num2str(RMSE1)))

axis([0 reals 0 max(RMSES(:,4)+1)])

subplot(4,2,2)

plot(1:reals,RMSES(:,1),'x')
xlabel('Realization')
ylabel('n')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
plot(index1,npatches,'ro')

title(strcat('npatches=',num2str(npatches)))

axis([0 reals 0 20])


subplot(4,2,3)

plot(1:reals,RMSES(:,2),'x')
xlabel('Realization')
ylabel('onset_{thr}')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
plot(index1,onset_thr,'ro')

title(strcat('onset_thr=',num2str(onset_thr)))

axis([0 reals 0 100])

subplot(4,2,4)

plot(1:reals,RMSES(:,3),'x')
xlabel('Realization')
ylabel('q')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
plot(index1,q_hat,'ro')

title(strcat('q=',num2str(q_hat)))

axis([0 reals 0 1])


subplot(4,2,5)

plot(1:reals,PS(:,1),'x')
xlabel('Realization')
ylabel('r')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
plot(index1,r_hat,'ro')

title(strcat('r_hat=',num2str(r_hat)))

axis([0 reals 0 20])

subplot(4,2,6)

plot(1:reals,PS(:,2),'x')
xlabel('Realization')
ylabel('p')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
plot(index1,p_hat,'ro')

title(strcat('p_hat=',num2str(p_hat)))

axis([0 reals 0 1])

subplot(4,2,7)

plot(1:reals,PS(:,3),'x')
xlabel('Realization')
ylabel('a')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
plot(index1,a_hat,'ro')

title(strcat('a_hat=',num2str(a_hat)))

axis([0 reals 0 1])

subplot(4,2,8)

semilogy(1:reals,PS(:,4),'x')
xlabel('Realization')
ylabel('K')
set(gca,'FontSize', 24);
set(gcf,'color','white')
hold on
plot(index1,K_hat,'ro')

title(strcat('K_hat=',num2str(K_hat)))

axis([0 reals 0 100000000])

P0=P;

%%


close all

Phats=[];

goodnessfit=[];

modelfits=[];

SSstats1=[];

SSstats2=[];

data1=load(datafilename1)

data1=data1(epidemic_period,:);

data1(:,2)=data1(:,2)/scalar1;

I0=data1(1,2); % initial condition

if I0==0
    
    data1=data1(2:end,:);
    
end

data=data1(:,2);

[max1,index1]=max(data);

timevect=(data1(:,1))*DT;

r=P0(1);

p=P0(2);

a=P0(3);

K=P0(4);

z(1)=r;% initial guesses
z(2)=p;
z(3)=a;
z(4)=K;
z(5)=q;

switch flag1
    
    case 1 %GLM
        LB=[0  0 1 20 0]; 
        UB=[10  1 1 100000000 5];
    case 2 %GRM
        LB=[0  0 0 20 0]; 
        UB=[10  1 10 100000000  5];
    case 3 %Logistic
        LB=[0  1 1 20 0]; 
        UB=[10  1 1 100000000 5];
end


I0=data(1); % initial condition

hold on


options = optimoptions('lsqcurvefit','UseParallel',true,...
    'TolX',10^(-5),'TolFun',10^(-5));

[P,resnorm,residual,exitflag,output,lambda,J]=lsqcurvefit(@plotModifiedLogisticGrowthPatch1,z,timevect,data,LB,UB,options,I0,npatches,onset_thr,flag1);

npatches

P

r_hat=P(1);
p_hat=P(2);
a_hat=P(3);
K_hat=P(4);
q_hat=P(5);

IC=zeros(npatches,1);

IC(1,1)=I0;
IC(2:end,1)=1;

invasions=zeros(npatches,1);

invasions(1)=1;


[~,x]=ode23s(@modifiedLogisticGrowthPatch,timevect,IC,[],r_hat,p_hat,a_hat,K_hat,npatches,onset_thr,q_hat,flag1);

figure(10)

for j=1:npatches
    
    incidence1=[x(1,j);diff(x(:,j))];
    
    plot(timevect,incidence1)
    hold on
    
end

y=sum(x,2);

totinc=[y(1,1);diff(y(:,1))];

totinc(1)=totinc(1)-(npatches-1);

bestfit=totinc;

plot(timevect,totinc,'r')
hold on
plot(timevect,data,'bo')
xlabel('Time (days)');
ylabel('Data and best SIR fit')

Ptrue=[r_hat p_hat a_hat K_hat q_hat]

'generate simulation study to derive parameter uncertainty..'

% generate simulation study to derive parameter uncertainty

yi=cumsum(totinc);

Phatss=zeros(M,5);
SSEs=zeros(M,1);
curves=[];

fittedcurves=[];

for real=1:M
    
    yirData=zeros(length(yi),1);
    
    yirData(1)=yi(1);
    
    for t=2:length(yi)
        lambda=abs(yi(t)-yi(t-1));
        yirData(t,1)=poissrnd(lambda,1,1);
    end
    
    curves=[curves yirData];
    
    r=P0(1);
    
    p=P0(2);
    
    a=P0(3);
    
    K=P0(4);
    
    q=P0(5);
    
    
    z(1)=r;% initial guesses
    z(2)=p;
    z(3)=a;
    z(4)=K;
    z(5)=q;
    
    switch flag1
        
        case 1 %GLM
            LB=[0  0 1 20 0];
            UB=[10  1 1 100000000 5]; 
        case 2 %GRM
            LB=[0  0 0 20 0];
            UB=[10  1 10 100000000  5]; 
        case 3 %Logistic
            LB=[0  1 1 20 0]; 
            UB=[10  1 1 100000000 5]; 
    end
    
    I0=data(1); % initial condition
    
    hold on
    
    
    options = optimoptions('lsqcurvefit','UseParallel',true,...
        'TolX',10^(-5),'TolFun',10^(-5));
    
    [P,resnorm,residual,exitflag,output,lambda,J]=lsqcurvefit(@plotModifiedLogisticGrowthPatch1,z,timevect,yirData,LB,UB,options,I0,npatches,onset_thr,flag1);
    
    
    fittedCurve1=residual+yirData;
    
    fittedcurves=[fittedcurves fittedCurve1];
    
    % P is the vector with the estimated parameters
    
    r_hat=P(1);
    p_hat=P(2);
    a_hat=P(3);
    K_hat=P(4);
    q_hat=P(5);
    
    Phatss(real,:)=P;
    
end


figure(100)
subplot(2,4,1)
hist(Phatss(:,1))
xlabel('r')
subplot(2,4,2)
hist(Phatss(:,2))
xlabel('p')
subplot(2,4,3)
hist(Phatss(:,4))
xlabel('K')
subplot(2,4,4)
hist(Phatss(:,5))
xlabel('q')


% plot model fit and uncertainty

param_r_time2=[];
param_p_time2=[];

subplot(2,4,[5 6 7 8])
plot(timevect,curves,'c-')
hold on

line1=plot(timevect,plims(curves',0.5),'r-')
set(line1,'LineWidth',2)

[max1,indexpeak1]=max(data1(:,2));

indexpeak1=indexpeak1+3;

line1=plot(timevect,data1(:,2),'bo')
set(line1,'LineWidth',2)

hold on

param_r=[mean(Phatss(:,1)) plims(Phatss(:,1),0.025) plims(Phatss(:,1),0.975)];

param_p=[mean(Phatss(:,2)) plims(Phatss(:,2),0.025) plims(Phatss(:,2),0.975)];

param_a=[mean(Phatss(:,3)) plims(Phatss(:,3),0.025) plims(Phatss(:,3),0.975)];

param_K=[mean(Phatss(:,4)) plims(Phatss(:,4),0.025) plims(Phatss(:,4),0.975)];

param_q=[mean(Phatss(:,5)) plims(Phatss(:,5),0.025) plims(Phatss(:,5),0.975)];


cad1=strcat('r=',num2str(param_r(end,1),2),'(95%CI:',num2str(param_r(end,2),2),',',num2str(param_r(end,3),2),')')
cad2=strcat('p=',num2str(param_p(end,1),2),'(95%CI:',num2str(param_p(end,2),2),',',num2str(param_p(end,3),2),')')
cad3=strcat('a=',num2str(param_a(end,1),2),'(95%CI:',num2str(param_a(end,2),2),',',num2str(param_a(end,3),2),')')
cad4=strcat('K=',num2str(param_K(end,1),2),'(95%CI:',num2str(param_K(end,2),2),',',num2str(param_K(end,3),2),')')
cad5=strcat('q=',num2str(param_q(end,1),2),'(95%CI:',num2str(param_q(end,2),2),',',num2str(param_q(end,3),2),')')


title(strcat('\fontsize{14}',cad1,';',cad2,';',cad3,';',cad5))

xlabel('\fontsize{16}Time (days)');
ylabel('\fontsize{16}Case incidence')
