%%%%
%Fitting code for sub-epidemic model associated with paper "A novel sub-epidemic modeling framework for short-term forecasting epidemic waves"
%Chowell et al. BMC Medicine 2019
%%%%

clear
close all

global invasions


%SARS SINGAPOPRE

load modifiedLogisticPatch-trained-70-SARS-Singapore-flag-1.mat



%% plot best fit and parameter uncertainties

param_r=[mean(Phatss(:,1)) plims(Phatss(:,1),0.025) plims(Phatss(:,1),0.975)];

param_p=[mean(Phatss(:,2)) plims(Phatss(:,2),0.025) plims(Phatss(:,2),0.975)];

param_a=[mean(Phatss(:,3)) plims(Phatss(:,3),0.025) plims(Phatss(:,3),0.975)];

param_K=[mean(Phatss(:,4)) plims(Phatss(:,4),0.025) plims(Phatss(:,4),0.975)];

param_q=[mean(Phatss(:,5)) plims(Phatss(:,5),0.025) plims(Phatss(:,5),0.975)];


cad1=strcat('r=',num2str(param_r(end,1),2),'(95%CI:',num2str(param_r(end,2),2),',',num2str(param_r(end,3),2),')');
cad2=strcat('p=',num2str(param_p(end,1),2),'(95%CI:',num2str(param_p(end,2),2),',',num2str(param_p(end,3),2),')')
cad3=strcat('a=',num2str(param_a(end,1),2),'(95%CI:',num2str(param_a(end,2),2),',',num2str(param_a(end,3),2),')')
cad4=strcat('K=',num2str(param_K(end,1),3),'(95%CI:',num2str(param_K(end,2),3),',',num2str(param_K(end,3),3),')')
cad5=strcat('q=',num2str(param_q(end,1),3),'(95%CI:',num2str(param_q(end,2),3),',',num2str(param_q(end,3),3),')')


close all
figure(100)
timevect=(data1(:,1))*DT;

subplot(2,4,1)
hist(Phatss(:,1))
xlabel('r')
ylabel('Frequency')
title(cad1)

hold on

line2=[param_r(1,2) 10;param_r(1,3) 10];

line1=plot(line2(:,1),line2(:,2),'r--')
set(line1,'LineWidth',2)

axis([param_r(1,2)-0.05 param_r(1,3)+0.05 0 120])

set(gca,'FontSize', 16);
set(gcf,'color','white')


subplot(2,4,2)
hist(Phatss(:,2))
xlabel('p')
hold on
title(cad2)

line2=[param_p(1,2) 10;param_p(1,3) 10];

line1=plot(line2(:,1),line2(:,2),'r--')
set(line1,'LineWidth',2)

axis([param_p(1,2)-0.025 param_p(1,3)+0.025 0 120])

set(gca,'FontSize', 16);
set(gcf,'color','white')


subplot(2,4,3)
hist(Phatss(:,4))
xlabel('K')
hold on
title(cad4)

line2=[param_K(1,2) 10;param_K(1,3) 10];

line1=plot(line2(:,1),line2(:,2),'r--')
set(line1,'LineWidth',2)

axis([param_K(1,2)-50 param_K(1,3)+50 0 120])

set(gca,'FontSize', 16);
set(gcf,'color','white')

subplot(2,4,4)
hist(Phatss(:,5))
xlabel('q')
hold on
title(cad5)

line2=[param_q(1,2) 10;param_q(1,3) 10];

line1=plot(line2(:,1),line2(:,2),'r--')
set(line1,'LineWidth',2)

axis([max(param_q(1,2)-0.1,0) param_q(1,3)+0.1 0 200])

set(gca,'FontSize', 16);
set(gcf,'color','white')


% plot model fit and uncertainty


subplot(2,4,5)
plot(timevect,curves,'c-')
hold on

line1=plot(timevect,bestfit,'r-')

set(line1,'LineWidth',2)

line1=plot(timevect,plims(curves',0.025),'r--')
set(line1,'LineWidth',2)

line1=plot(timevect,plims(curves',0.975),'r--')
set(line1,'LineWidth',2)

line1=plot(timevect,data1(:,2),'ko')
set(line1,'LineWidth',2)

hold on


% 95% prediction coverage
coverage1=find(data1(:,2)>=plims(curves',0.025)' & data1(:,2)<=plims(curves',0.975)');


xlabel('Time (days)')
ylabel('Case incidence')

axis([timevect(1) timevect(end)+1 0 max(data(:,2))*1.3])

set(gca,'FontSize', 16);
set(gcf,'color','white')

%goodness of fit

MAE=mean(abs(bestfit-data1(:,2)))

MSE=mean((bestfit-data1(:,2)).^2)

RMSE=sqrt(MSE)

Lt=plims(curves',0.025)';
Ut=plims(curves',0.975)';

coverage1=find(data1(:,2)>=Lt & data1(:,2)<=Ut);

'prediction coverage (%)='
PI=100*(length(coverage1)./length(data1(:,2)))

MIS=mean([(Ut-Lt)+(2/0.05).*(Lt-data1(:,2)).*(data1(:,2)<Lt)+(2/0.05)*(data1(:,2)-Ut).*(data1(:,2)>Ut)])


% residuals

subplot(2,4,[7 8])

resid1=bestfit-data1(:,2);

stem(timevect,resid1,'b')
hold on

xlabel('Time (days)')
ylabel('Residuals')

axis([timevect(1) timevect(end)+1 min(resid1)-1 max(resid1)+1])

set(gca,'FontSize', 16);
set(gcf,'color','white')


%% plot sub-epidemic profile

color1=['r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';];


% generate forecast curves from each bootstrap realization
for realization=1:M
    
    
    r_hat=Phatss(realization,1);
    p_hat=Phatss(realization,2);
    a_hat=Phatss(realization,3);
    K_hat=Phatss(realization,4);
    q_hat=Phatss(realization,5);
    
    IC=zeros(npatches,1);
    
    IC(1,1)=data1(1,2);
    IC(2:end,1)=1;
    
    invasions=zeros(npatches,1);
 
    invasions(1)=1;

    
    
    [~,x]=ode23s(@modifiedLogisticGrowthPatch,timevect,IC,[],r_hat,p_hat,a_hat,K_hat,npatches,onset_thr,q_hat,flag1);
    
    subplot(2,4,6)
    
    for j=1:npatches
        
        incidence1=[x(1,j);diff(x(:,j))];
        
        plot(timevect,incidence1,color1(j,:))
        
        hold on
        
    end
    
    y=sum(x,2);
    
    totinc=[y(1,1);diff(y(:,1))];
    
    totinc(1)=totinc(1)-(npatches-1);
    
    bestfit=totinc;
    
    gray1=gray(10);
    
    plot(timevect,totinc,'color',gray1(7,:))
    %hold on
    plot(timevect,data1(:,2),'ko')
    
    
    dist1=1;
    factor1=1;
    
    
end

xlabel('\fontsize{16}Time (days)');
ylabel('\fontsize{16}Case incidence')

line1=plot(data(:,1)*DT,data(:,2),'ko')
set(line1,'LineWidth',2)

axis([timevect(1) timevect(end)+1 0 max(data(:,2))*1.3])

set(gca,'FontSize',16)
set(gcf,'color','white')

title('Sub-epidemic profile')


%% short-term forecasts

close all

outbreak=1;

switch outbreak
    
    case 1
        
        tfss2=[12 14 16 18 20 22 24 26 28 30]; % plague
        
    case 2
        tfss2=[8 10 12 14 16 18 20 22 24 26]; %Ebola DRC
        
    case 3
        tfss2=[15 20 25 30 35 40 45 50 55 60]; %SARS in Singapore
        
end


for w=1:length(tfss2)
    
    %%%Plague
    load(strcat('modifiedLogisticPatch-trained-',num2str(tfss2(w)),'-curve-plague-Madagascar-wave2-flag-1.mat'))
    
    
    %%%%Ebola in DRC
    %load(strcat('modifiedLogisticPatch-trained-',num2str(tfss2(w)),'-Ebola-DRC-April9-flag-1.mat'));
    
    
    %%%%SARS
    %load(strcat('modifiedLogisticPatch-trained-',num2str(tfss2(w)),'-SARS-Singapore-flag-1.mat'))
    
    
    timevect=(data1(:,1))*DT;
    
    % forecast horizon
    forecastingperiod=6; % number of data points ahead
    
    
    timevect2=(0:t_window(end)-1+forecastingperiod)*DT;
    
    curvesforecasts1=[];
    
    % vector to store forecast prediction curves
    curvesforecasts2=[];
    
    color1=['r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';'r-';'b-';'g-';'m-';'c-';'k-';'y-';];
    
    
    % generate forecast curves from each bootstrap realization
    for realization=1:M
        
        
        r_hat=Phatss(realization,1);
        p_hat=Phatss(realization,2);
        a_hat=Phatss(realization,3);
        K_hat=Phatss(realization,4);
        q_hat=Phatss(realization,5);
        
        IC=zeros(npatches,1);
        
        IC(1,1)=data1(1,2);
        IC(2:end,1)=1;
        
        invasions=zeros(npatches,1);
        
        invasions(1)=1;

        [~,x]=ode23s(@modifiedLogisticGrowthPatch,timevect2,IC,[],r_hat,p_hat,a_hat,K_hat,npatches,onset_thr,q_hat,flag1);
        
        figure(10)
        
        subplot(5,2,w)
        
        
        
        for j=1:npatches
            
            incidence1=[x(1,j);diff(x(:,j))];
            
            plot(timevect2,incidence1,color1(j,:))
            
            hold on
            
        end
        
        y=sum(x,2);
        
        totinc=[y(1,1);diff(y(:,1))];
        
        totinc(1)=totinc(1)-(npatches-1);
        
        bestfit=totinc;
        
        gray1=gray(10);
        
        plot(timevect2,totinc,'color',gray1(7,:))
        %hold on
        plot(timevect,data1(:,2),'ko')
        
        
        dist1=1;
        factor1=1;
        
        curvesforecasts1=[curvesforecasts1 totinc];
        
        forecasts2=AddPoissonError(cumsum(totinc),20,dist1,factor1);
        
        curvesforecasts2=[curvesforecasts2 forecasts2];
        
        
    end
    
    
    xlabel('\fontsize{24}Time (days)');
    ylabel('\fontsize{24}Case incidence')
    
    line1=plot(data(:,1)*DT,data(:,2),'ko')
    set(line1,'LineWidth',2)
    
    % plot vertical line separating calibration versus forecast periods
    line2=[timevect(end) 0;timevect(end) max(data(:,2))*1.5];
    
    switch outbreak
        case 1
            axis([0 500 0 150]) % plague Madagascar
            
        case 2
            axis([0 260 0 80]) %Ebola DRC
            
        case 3
            axis([0 70 0 40]) %SARS
            
    end
    
    line1=plot(line2(:,1),line2(:,2),'k--')
    set(line1,'LineWidth',2)
    
    set(gca,'FontSize',24)
    set(gcf,'color','white')
    
    
    figure(11)
    
    %subplot(1,3,w)
    switch outbreak
        case 2.5
            
            subplot(2,2,w)
            
        otherwise
            subplot(5,2,w)
            
    end
    
    
    hold on
    
    %line1=plot(timevect2,plims(curvesforecasts2',0.5),'r-')
    line1=plot(timevect2,mean(curvesforecasts2,2),'r-')
    
    set(line1,'LineWidth',2)
    
    line1=plot(timevect2,plims(curvesforecasts2',0.025),'r--')
    set(line1,'LineWidth',2)
    
    line1=plot(timevect2,plims(curvesforecasts2',0.975),'r--')
    set(line1,'LineWidth',2)
    
    
    gray1=gray(10);
    
    if 0
        % plot forecast mean curves and 95% confidence intervals
        plot(timevect2,curvesforecasts1,'color',gray1(7,:))
        hold on
        
        line1=plot(timevect2,plims(curvesforecasts1',0.5),'r-')
        set(line1,'LineWidth',2)
        
        line1=plot(timevect2,plims(curvesforecasts1',0.025),'r:')
        set(line1,'LineWidth',2)
        
        line1=plot(timevect2,plims(curvesforecasts1',0.975),'r:')
        set(line1,'LineWidth',2)
    end
    
    
    xlabel('\fontsize{24}Time (days)');
    ylabel('\fontsize{24}Case incidence')
    
    % plot time series data
    line1=plot(data(:,1)*DT,data(:,2),'ko')
    set(line1,'LineWidth',2)
    
    % plot vertical line separating calibration versus forecast periods
    line2=[timevect(end) 0;timevect(end) max(data(:,2))*1.5];
        
    switch outbreak
        case 1
            axis([0 500 0 150]) % plague Madagascar
            
        case 2
            axis([0 260 0 80]) %Ebola DRC
            
        case 3 %SARS
            axis([0 70 0 40])
            
    end
    
    line1=plot(line2(:,1),line2(:,2),'k--')
    set(line1,'LineWidth',2)
    
    set(gca,'FontSize',24)
    set(gcf,'color','white')

end
